#ifndef BS_DETECTION_WIDGET_H
#define BS_DETECTION_WIDGET_H

#include <QGraphicsView>
#include <QColor>
#include <QImage>
#include <QWidget>
#include <QVector>
#include <fstream>
#include "bsdetector.h"
#include "bsaccumulatorview.h"
#include "bsstructureview.h"
#include "bsprofileview.h"

using namespace std;


/** 
 * @class BSDetectionWidget bsdetectionwidget.h
 * \brief Segment extraction view and controller.
 * \author {P. Even and B. Kerautret}
 */
class BSDetectionWidget : public QWidget 
{
  Q_OBJECT


public:

  /**
   * \brief Creates a segment extraction widget.
   */
  BSDetectionWidget (QWidget *parent = 0);

  /**
   * \brief Deletes the segment extraction widget.
   */
  ~BSDetectionWidget ();

  /**
   * \brief Opens the image to be processed.
   * @param type Name of the image file to open.
   * @param type Gradient extraction method.
   */
  QSize openImage (const QString &fileName, int type = 0);

  /**
   * \brief Builds and returns the image bitmap.
   */
  int **getBitmap (const QImage &image);
  
  /**
   * \brief Updates the Qt widget display.
  void paint (QPainter *painter,
              const QStyleOptionGraphicsItem *option, QWidget *widget);
   */

  /**
   * \brief Saves the augmented image with extraction results.
   */
  bool saveAugmentedImage (const QString &fileName, const char *fileFormat);

  /**
   * \brief Requires the accumulation window closure.
   */
  void closeAccuAnalyzer ();

  /**
   * \brief Requires the pixel analysis window closure.
   */
  void closePixelAnalyzer ();

  /**
   * \brief Requires the profile analysis window closure.
   */
  void closeProfileAnalyzer ();

  /**
   * \brief Switches the pixel display window on or off.
   */
  void switchPixelAnalyzer ();

  /**
   * \brief Switches the accumulator display window on or off.
   */
  void switchAccuAnalyzer ();

  /**
   * \brief Switches the profile display window on or off.
   */
  void switchProfileAnalyzer ();

  /**
   * \brief Switches the extraction result display on or off.
   */
  inline void switchVerbose () { verbose = ! verbose; }

  /**
   * \brief Runs a comparative test.
   */
  void alternateTest ();

  /**
   * \brief Runs a performance test.
   * Displays the time spent for 1000 detections under the present stroke.
   */
  void performanceTest ();

  /**
   * \brief Runs a local test (to be adapted).
   */
  void localTest ();


public slots:
  /**
   * \brief Clears the widget drawing.
   */
  void clearImage ();


protected:
  /**
   * \brief Updates the widget drawing.
   */
  void paintEvent (QPaintEvent *event);

  /**
   * \brief Processes mouse press events.
   */
  void mousePressEvent (QMouseEvent *event);

  /**
   * \brief Processes mouse release events.
   */
  void mouseReleaseEvent (QMouseEvent *event);

  /**
   * \brief Processes move release events.
   */
  void mouseMoveEvent (QMouseEvent *event);

  /**
   * \brief Processes key press events.
   */
  void keyPressEvent (QKeyEvent *event);


private:
 
  /** Sensitiveness of segment max width parameter. */
  static const int MAX_WIDTH_TUNING;
  /** Sensitiveness of detection lacks parameter. */
  static const int DETECTION_LACKS_TUNING;
  /** Default value for pen width. */
  static const int DEFAULT_PEN_WIDTH;


  /** Initial scan end points */
  Pt2i p1, p2;
  /** Flag indicating if the stroke is completely defined. */
  bool fixed;
  /** Flag indicating whether extraction result should be displayed. */
  bool verbose;

  /** Activation of alternate comparative tests (F8). */
  int alternate;

  /** Index of the last blurred segment displayed in a multi-selection. */
  int instanceOfAll;


  /** Presently loaded image. */
  QImage loadedImage;
  /** Present image augmented with processed data. */
  QImage augmentedImage;
  /** Gradient map of the loaded picture. */
  VMap *gMap;
  /** Width of the present image. */
  int width;
  /** Height of the present image. */
  int height;
  /** Blurred segment detector. */
  BSDetector detector;

  /** Aggregation of segment extraction results with initial conditions. */
  struct ExtractedSegment
  {
    /** Extracted blurred segment. */
    BlurredSegment *bs;
    /** Selected strock start point. */
    Pt2i p1;
    /** Selected stroke end point. */
    Pt2i p2;
  };

  /** List of registred blurred segments. */
  vector<ExtractedSegment> extractedSegments;

  /** Scanned profile graphics view. */
  BSProfileView *profileview;
  /** Filter accumulator view. */
  BSAccumulatorView *accuview;
  /** Blurred segment contents view. */
  BSStructureView *strucview;



  /**
   * \brief Draws a list of points with the given color.
   */
  void drawListOfPixels (vector<Pt2i> vectPixels,
                         QColor color, QPainter &painter);

  /**
   * \brief Draws a list of image pixels.
   */
  void drawListOfPixels (vector<Pt2i> vectPixels,
                       QPainter &painter);

  /**
   * \brief Draws the line joining two points.
   */
  void drawLine (const Pt2i from, const Pt2i to,
                 QColor color, QPainter &painter);

  /**
   * \brief Registers the last extracted blurred segment.
   */
  void storeExtractedSegment ();

  /**
   * \brief Displays the registred blurred segments.
   */
  void displayExtractedSegments ();

  /**
   * \brief Clears off the already extracted segments.
   */
  void clearExtractedSegments ();

  /**
   * \brief Displays the last extraction result.
   */
  void displayExtractionResult ();

  /**
   * \brief Detects a blurred segment under the selected stroke.
   * @param withAllDisplay Indicates if all information about the extraction
   *                       should be displayed.
   */
  void extract (bool withAllDisplay);

  /**
   * \brief Detects and displays a blurred segment under the selected stroke.
   */
  void extract ();

  /**
   * \brief Detects and displays all the blurred segment in the picture.
   */
  void extractAll ();

  /**
   * \brief Highlights the next blurred segment in multi-selection mode.
   */
  void displayNextOfAll ();

};
#endif
